<?php

namespace Drupal\cl_block\Form;

use Drupal\cl_components\Component\Component;
use Drupal\cl_components\Component\ComponentDiscovery;
use Drupal\cl_components\Component\ComponentMetadata;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Component Libraries: Blocks settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactoryInterface $config_factory, private readonly ComponentDiscovery $componentDiscovery, private readonly BlockManagerInterface $blockManager) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $component_discovery = $container->get(ComponentDiscovery::class);
    assert($component_discovery instanceof ComponentDiscovery);
    return new static(
      $container->get('config.factory'),
      $component_discovery,
      $container->get('plugin.manager.block')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cl_block_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['cl_block.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $settings = $this->config('cl_block.settings')->get();
    $types = $form_state->getValue('types') ?? $form_state->getUserInput['types'] ?? $settings['types'];
    $statuses = $form_state->getValue('statuses') ?? $form_state->getUserInput['statuses'] ?? $settings['statuses'];
    // I am unsure why we need this, but the component discovery is not
    // initialized in ajax callbacks sometimes.
    $discovery = isset($this->componentDiscovery) ? $this->componentDiscovery : \Drupal::service(ComponentDiscovery::class);
    $components = $discovery->findAllWithFilters([], [], $types, $statuses);
    $options = array_map(static fn (Component $component) => $component->getMetadata()->getName(), $components);
    $form['filters'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Filters'),
    ];
    $form['filters']['info'] = [
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#value' => $this->t('Set a combination of filters to control the list of components that will become blocks.'),
    ];
    $form['filters']['types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Component Types'),
      '#description' => $this->t('Only components of these types will show in the component list.'),
      '#options' => [
        ComponentMetadata::COMPONENT_TYPE_ATOM => $this->t('Atom'),
        ComponentMetadata::COMPONENT_TYPE_MOLECULE => $this->t('Molecule'),
        ComponentMetadata::COMPONENT_TYPE_ORGANISM => $this->t('Organism'),
      ],
      '#default_value' => $settings['types'],
      '#ajax' => [
        'wrapper' => 'refinement-wrapper',
        'effect' => 'fade',
        'callback' => '::onChangeAjaxCallback',
      ],
    ];
    $form['filters']['statuses'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Component Status'),
      '#description' => $this->t('Only components of these status will show in the component list.'),
      '#options' => [
        ComponentMetadata::COMPONENT_STATUS_READY => $this->t('Ready'),
        ComponentMetadata::COMPONENT_STATUS_BETA => $this->t('Beta'),
        ComponentMetadata::COMPONENT_STATUS_WIP => $this->t('Work in progress'),
        ComponentMetadata::COMPONENT_STATUS_DEPRECATED => $this->t('Deprecated'),
      ],
      '#default_value' => $settings['statuses'],
      '#ajax' => [
        'wrapper' => 'refinement-wrapper',
        'effect' => 'fade',
        'callback' => '::onChangeAjaxCallback',
      ],
    ];
    $form['filters']['refine'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Refine selection'),
      '#prefix' => '<div id="refinement-wrapper">',
      '#suffix' => '</div>',
    ];
    $form['filters']['refine']['forbidden'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Forbidden components'),
      '#description' => $this->t('When listing the blocks for CL Components, all components will be displayed except the components selected here. This is used to limit the components available for a given content type.'),
      '#options' => $options,
      '#default_value' => $settings['forbidden'],
      '#attributes' => ['class' => ['long-checkboxes-list']],
    ];
    $form['filters']['refine']['allowed'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Allowed components'),
      '#description' => $this->t('When listing the blocks for CL Components, only the components selected here will be displayed as an option. <strong>NOTE:</strong> "forbidden" components will not be shown even when allowed here.'),
      '#options' => $options,
      '#default_value' => $settings['allowed'],
      '#attributes' => ['class' => ['long-checkboxes-list']],
    ];
    $form['filters']['#attached']['library'][] = 'cl_block/settings';
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $forbidden = array_keys(array_filter($form_state->getValue('forbidden')));
    $allowed = array_keys(array_filter($form_state->getValue('allowed')));
    $both_lists = array_intersect($forbidden, $allowed);
    if (!empty($both_lists)) {
      $message = $this->t('The following components are added to both forbidden and allow lists. Please review: %components', ['%components' => implode(', ', $both_lists)]);
      $form_state->setErrorByName('forbidden', $message);
      $form_state->setErrorByName('allowed', $message);
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $types = array_keys(array_filter($form_state->getValue('types')));
    $statuses = array_keys(array_filter($form_state->getValue('statuses')));
    $forbidden = array_keys(array_filter($form_state->getValue('forbidden')));
    $allowed = array_keys(array_filter($form_state->getValue('allowed')));
    $this->config('cl_block.settings')
      ->set('types', $types)
      ->set('statuses', $statuses)
      ->set('forbidden', $forbidden)
      ->set('allowed', $allowed)
      ->save();
    // Refresh the list of blocks after saving settings.
    $manager = isset($this->blockManager) ? $this->blockManager : \Drupal::service('plugin.manager.block');
    $manager->clearCachedDefinitions();
    parent::submitForm($form, $form_state);

  }

  /**
   * Ajax callback to regenerate the list of components.
   */
  public function onChangeAjaxCallback(array &$element, FormStateInterface $form_state) {
    return NestedArray::getValue($element, ['filters', 'refine']);
  }

}
