<?php

namespace Drupal\cl_block\Plugin\Filter;

use Drupal\cl_block\Render\ComponentBlockRenderer;
use Drupal\Component\Utility\Html;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Markup;
use Drupal\embed\DomHelperTrait;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Symfony\Component\DependencyInjection\ContainerInterface;


/**
 * Provides a filter to display embedded entities based on data attributes.
 *
 * @Filter(
 *   id = "cl_block",
 *   title = @Translation("Embedded CL Components"),
 *   description = @Translation("Embeds CL Components using data attributes:
 *   data-component-id, data-component-variant and data-component-settings.
 *   Should usually run as the last filter, since it does not contain user
 *   input."), type =
 *   Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_REVERSIBLE, weight =
 *   100,
 * )
 */
class ClComponentEmbed extends FilterBase implements ContainerFactoryPluginInterface {

  use DomHelperTrait;

  /**
   * The renderer service.
   *
   * @var \Drupal\cl_block\Render\ComponentBlockRenderer
   */
  readonly protected ComponentBlockRenderer $renderer;

  /**
   * Constructs a ElementEmbed object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\cl_block\Render\ComponentBlockRenderer $renderer
   *   The renderer.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ComponentBlockRenderer $renderer) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $renderer = $container->get(ComponentBlockRenderer::class);
    assert($renderer instanceof ComponentBlockRenderer);
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $renderer,
    );
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    $result = new FilterProcessResult($text);
    if (!str_contains($text, 'cl-component')) {
      return $result;
    }

    $dom = Html::load($text);
    $xpath = new \DOMXPath($dom);

    $xpath_query = '//cl-component[@data-component-id and @data-component-settings]';
    foreach ($xpath->query($xpath_query) as $node) {
      assert($node instanceof \DOMElement);
      $element_output = '';
      $data = $this->getNodeAttributesAsArray($node);
      $component_id = $data['data-component-id'] ?? '';
      $collected_attachments = [];
      $element_output = $this->renderer->renderComponentFromId(
        $component_id,
        $data['data-component-settings']['variant'] ?? '',
        $data['data-component-settings']['data'] ?? [],
        $data['data-component-settings']['twig_blocks'] ?? [],
        [] // @todo how do we derive token replacement data?
      );
      $result->addAttachments($collected_attachments);
      $this->replaceNodeContent($node, $element_output instanceof Markup ? (string) $element_output : $element_output);
    }

    $result->setProcessedText(Html::serialize($dom));
    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function tips($long = FALSE) {
    return $this->t('You can embed CL Components.');
  }

}
