<?php

namespace Drupal\cl_block\Render;

use Drupal\cl_components\Component\Component;
use Drupal\cl_components\Component\ComponentDiscovery;
use Drupal\cl_components\Exception\ComponentNotFoundException;
use Drupal\cl_components\Exception\TemplateNotFoundException;
use Drupal\Component\Plugin\Exception\ContextException;
use Drupal\Component\Render\MarkupInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\RefinableCacheableDependencyInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Render\Markup;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Utility\Token;

class ComponentBlockRenderer {

  public function __construct(protected readonly RendererInterface $renderer, protected readonly Token $token, protected readonly ComponentDiscovery $componentDiscovery) {
  }

  /**
   * Builds a component with all the necessary context for a block.
   *
   * @param string $component_id
   *   The component ID.
   * @param string $variant
   *   The component variant.
   * @param array $context
   *   The Twig context.
   * @param array $twig_blocks
   *   The content for the twig blocks.
   * @param array $token_data
   *   The token data for token replacements.
   *
   * @return array
   *   The output render array.
   */
  public function buildFromId(
    string $component_id,
    string $variant,
    array $context,
    array $twig_blocks,
    array $token_data = []
  ): array {
    try {
      $component = $this->componentDiscovery->find($component_id);
    }
    catch (ComponentNotFoundException $e) {
      return [];
    }
    return $this->build(
      $component,
      $variant,
      $context,
      $twig_blocks,
      $token_data
    );
  }

  /**
   * Renders a component with all the necessary context for a block.
   *
   * @param \Drupal\cl_components\Component\Component $component
   *   The component.
   * @param string $variant
   *   The component variant.
   * @param array $context
   *   The Twig context.
   * @param array $twig_blocks
   *   The content for the twig blocks.
   * @param array $token_data
   *   The token data for token replacements.
   * @param \Drupal\Core\Render\BubbleableMetadata $bubbleable_metadata
   *   The metadata.
   *
   * @return array
   *   The output render array.
   */
  public function build(
    Component $component,
    string $variant,
    array $context,
    array $twig_blocks,
    array $token_data = []
  ): array {
    $bubbleable_metadata = new BubbleableMetadata();
    try {
      $template = $this->generateComponentTemplate(
        $component,
        $variant,
        $twig_blocks,
        $context,
        $token_data,
        $bubbleable_metadata
      );
    }
    catch (TemplateNotFoundException $e) {
      return [];
    }
    $build = [
      '#type' => 'inline_template',
      '#template' => $template,
    ];
    $bubbleable_metadata->applyTo($build);
    return $build;
  }

  /**
   * Renders a component.
   */
  public function renderComponentFromId(
    string $component_id,
    string $variant,
    array $context,
    array $twig_blocks,
    array $token_data,
  ): MarkupInterface {
    $build = $this->buildFromId(
      $component_id,
      $variant,
      $context,
      $twig_blocks,
      $token_data,
    );
    try {
      return $this->renderer->render($build);
    }
    catch (\Exception $e) {
      return Markup::create('');
    }
  }

  /**
   * Bubbles the cacheability metadata.
   *
   * This is useful when we manually serialize the render array for DOM
   * manipulation. This happens, at least, in the WYSIWYG replacements.
   *
   * @param \Drupal\Core\Render\BubbleableMetadata $metadata
   *   The metadata to bubble.
   */
  public function bubbleMetadata(BubbleableMetadata $metadata): void {
    $lib_build = [];
    $metadata->applyTo($lib_build);
    try {
      $this->renderer->render($lib_build);
    }
    catch (\Exception $e) {
      watchdog_exception('cl_block', $e);
    }
  }

  /**
   * Generates the template to render the component.
   *
   * @param \Drupal\cl_components\Component\Component $component
   *   The component.
   * @param mixed $variant
   *   The variant.
   * @param array $twig_blocks
   *   The contents of any potential embed blocks.
   * @param array $context
   *   The context data.
   * @param array $token_data
   *   The token data.
   * @param \Drupal\Core\Cache\CacheableMetadata $bubbleable_metadata
   *   The cacheable metadata.
   *
   * @return string
   *   The template.
   *
   * @throws \Drupal\cl_components\Exception\TemplateNotFoundException
   */
  private function generateComponentTemplate(
    Component $component,
    string $variant,
    array $twig_blocks,
    array $context,
    array $token_data,
    CacheableMetadata $bubbleable_metadata
  ): string {
    $metadata = $component->getMetadata();
    $component_path = $metadata->getPath();
    $template_name = $component->getTemplateName($variant);
    $template_path = $component_path . DIRECTORY_SEPARATOR . $template_name;
    $template = '{# This template was dynamically generated by cl_block #}' . PHP_EOL;
    // Replace possible tokens in the context mapping.
    $context = $this->replaceTokensRecursively($context, $token_data, $bubbleable_metadata);
    $template .= empty($context)
      ? sprintf('{%% embed \'%s\' %%}', $template_path)
      : sprintf('{%% embed \'%s\' with %s %%}', $template_path, Json::encode($context));
    $template .= PHP_EOL;
    $options = ['clear' => TRUE];
    try {
      // If there is a language in the context of the block, use it.
      $language = $token_data['language'] ?? NULL;
      if ($language instanceof LanguageInterface) {
        $options['langcode'] = $language->getId();
      }
    }
    catch (ContextException $e) {
      // Intentionally left blank.
    }
    foreach ($twig_blocks as $block_name => $block_value) {
      $bl_val = $this->token->replace(
        $block_value['value'],
        $token_data,
        $options,
        $bubbleable_metadata
      );
      $bl_format = is_array($block_value) ? $block_value['format'] : 'full_html_with_twig';
      $block_build = [
        '#type' => 'processed_text',
        '#text' => $bl_val,
        '#format' => $bl_format,
      ];
      try {
        $value = $this->renderer->render($block_build);
      }
      catch (\Exception $e) {
        $value = $bl_val;
      }
      // Save the collected attachments before we do the stringification.
      $twig_block_metadata = BubbleableMetadata::createFromRenderArray($block_build);
      $bubbleable_metadata = $bubbleable_metadata->merge($twig_block_metadata);
      $template .= "  {% block $block_name %}" . PHP_EOL
        . "    $value" . PHP_EOL
        . "  {% endblock %}" . PHP_EOL;
    }
    $template .= '{% endembed %}' . PHP_EOL;
    return $template;
  }

  /**
   * Replaces tokens recursively in a data structure.
   *
   * @param array $context
   *   The context.
   * @param array $token_data
   *   The token data.
   * @param \Drupal\Core\Cache\RefinableCacheableDependencyInterface $bubbleable_metadata
   *   The bubbleable metadata.
   *
   * @return array
   *   The context with the tokens replaced.
   */
  private function replaceTokensRecursively(array $context, array $token_data, RefinableCacheableDependencyInterface $bubbleable_metadata): array {
    return array_map(
      function (mixed $item) use ($token_data, $bubbleable_metadata): mixed {
        if (is_string($item)) {
          return $this->token->replacePlain(
            $item,
            $token_data,
            ['clear' => TRUE],
            $bubbleable_metadata
          );
        }
        if (is_array($item)) {
          return $this->replaceTokensRecursively($item, $token_data, $bubbleable_metadata);
        }
        return $item;
      },
      $context
    );
  }

}
